/* iRoll:  Ball based self balancing human transport
   Aug 25, 2011
   Dale Heatherington
   www.wa4dsy.net/robot/
   
   This program requires PID2_v1, lowpass1, MsTimer2 and the SoftwareSerial librarys to be installed in the sketch directory
   
   This Code is licensed under a Creative Commons Attribution-ShareAlike 3.0 Unported License.
   
*/

#include <PID2_v1.h>
#include <SoftwareSerial.h>
#include <MsTimer2.h>
#include <Lowpass1.h>

#define rxPin 2
#define txPin 3
#define ledPin 7
#define SAMPLERATE 5

// Set up a new serial port for controlling the Sabertooth 2x25 in
//simplified serial mode.  0=stop  1-127=motor1 speed   128-255 = motor2
//Sabertooth 2x25 DIP switch setting = 101011  (1=ON)

SoftwareSerial MotorSerial =  SoftwareSerial(rxPin, txPin);
byte pinState = 0;

int n ;
char Fail;
float xspeed,yspeed, xtach,ytach, xmotor, ymotor;
float xAngle, yAngle, zeroTilt;


int xspd, yspd, lastxspd, lastyspd;
int debug1,debug2, pot0,pot1,pot2,pot3,pot4;
volatile  int dbctr, timer1, timer2;

//Closed loop motor speed control PID
PID xmotorPID(&xtach,&xmotor,&xspeed, 1.7, 0.55, 0,DIRECT);
PID ymotorPID(&ytach,&ymotor,&yspeed, 1.7, 0.55, 0,DIRECT);

//Balance loop PID
PID xBalancePID(&xAngle,&xspeed,&zeroTilt,0.5,0,0,DIRECT);
PID yBalancePID(&yAngle,&yspeed,&zeroTilt,0.5,0,0,DIRECT);

//Low pass filters for speed feedback loop.    (Not motor speed control)

Lowpass1 Xlowpass(0.7,SAMPLERATE);  // 0.7 HZ first order low pass filter 
Lowpass1 Ylowpass(0.7,SAMPLERATE);



void setup()
{
  
  delay(1000);  // Wait a second
    
  Serial.begin(9600);
  
  n = 0;
  
  // define pin modes for tx, rx, led pins:
  pinMode(rxPin, INPUT);
  pinMode(txPin, OUTPUT);
  pinMode(ledPin, OUTPUT);
  
  pinMode(4,OUTPUT);
  pinMode(5,OUTPUT);
  pinMode(6,OUTPUT);
  
  dbctr = 25;
  
  xspeed = 0;
  yspeed = 0;
  
  // Set the data rate for the SoftwareSerial port
  // I used software serial for motor control so I could use the hardware serial port for debugging
  
  MotorSerial.begin(9600);
  MotorSerial.print(0,BYTE);  //Motors stopped
  SetMotorSpeed(0,0);
  
  delay(4000);    //Wait 4 more seconds for analog circuits to stabilize
  
  //Setup motor speed control PID
  xmotorPID.SetOutputLimits(-120,120);
  ymotorPID.SetOutputLimits(-120,120);
  xmotorPID.SetSampleTime(SAMPLERATE);
  ymotorPID.SetSampleTime(SAMPLERATE);
  xmotorPID.SetMode(AUTOMATIC);
  ymotorPID.SetMode(AUTOMATIC);
 
 //Set up balance PID
 xBalancePID.SetSampleTime(SAMPLERATE);
 yBalancePID.SetSampleTime(SAMPLERATE);
 xBalancePID.SetOutputLimits(-110,110);
 yBalancePID.SetOutputLimits(-110,110);
 xBalancePID.SetMode(AUTOMATIC);
 yBalancePID.SetMode(AUTOMATIC);
  
  
  
  MsTimer2::set(10, timer100hz); // 10ms period 
  MsTimer2::start();
  
  timer1 = 100;
  timer2 = 1;
  zeroTilt = 0.0;
  
  xspeed = 0;
  yspeed = 0;
  Fail = 0;


  
}

void loop()
{float Ki,Kd,Kp;
 float balanceKp,balanceKi,balanceKd;
 float Xlp,Ylp,Xraw,Yraw, KK;
 
  
  readPots();  //Get analog pot values
  
  balanceKp = ((float)pot0) / 200;  //Pot0 controls Kp proportional
  balanceKi = (float)pot1 / 1024;   //Pot1 controls Ki integral
  balanceKd = (float)pot2 / 150;    //Pot2 controls Kd 
  
  xBalancePID.SetTunings(balanceKp, balanceKi, balanceKd);   //Apply pot values to PID tuning
  yBalancePID.SetTunings(balanceKp, balanceKi, balanceKd);
  
  ytach = (analogRead(2)/4) - 128;    //Read the tachs to get motor speed
  ytach = ytach * 2;
  xtach = (analogRead(3)/4) - 128;
  xtach = xtach * 2;
  
  Xlp = 0;
  Ylp = 0;   
   

//Get X and Y speed and lowpass filter the values
  Xlp = Xlowpass.filter(xtach);      //Low pass filter the tachs  
  Xlp = Xlp * ((float)pot4/1024);   //Use pot4 to adjust amplitude of filtered tach data  (Speed Feedback pot)
  Ylp = Ylowpass.filter(ytach);  
  Ylp = Ylp * ((float)pot4/1024);
  
  xAngle = analogRead(1)/4 - 128;      // Read X & Y tilt angles
  yAngle = analogRead(0)/4 - 128;  
  
  //Check for excessive tilt and set or clear "Fail"
  if((abs(xAngle) < (float)50) && (abs(yAngle) < (float)50)) Fail = 0; 
  if((abs(xAngle) >= (float)126) || (abs(yAngle) >= (float)126)) Fail = 1;   //FAIL=TRUE if it falls over.
  
  xAngle = xAngle - Xlp;    //Add lowpassed filtered speed to the tilt angle to prevent runaway and eventual tip over.
  yAngle = yAngle - Ylp;     //Note Pot4 controls how much feedback.
  
  // Do balance PID loop
  xBalancePID.Compute();    //xAngle-zerotilt -> PID -> xspeed
  yBalancePID.Compute();
  
  // Do motor speed control PID loop  
  ymotorPID.Compute();    
  xmotorPID.Compute();      // xtach-xspeed -> PID -> xmotor
  
  //Convert PID outpout to integers
  xspd = (int)xmotor;
  yspd = (int)ymotor; 
  
  if(Fail == 1) { xspd = 0; yspd = 0; }   //Stop motors if it falls over
  digitalWrite(ledPin,1);      //Set speed to zero
  
  //Send PID motor control speed values to Sabertooth 2x25 controller via software serial port if they changed since last time.
  
   if((xspd != lastxspd) || (yspd != lastyspd)) { SetMotorSpeed(xspd,yspd);  lastyspd = yspd; lastxspd = xspd; }
  digitalWrite(ledPin,0);
  
  /* Debug code */
  //Debug(xtach,xspeed,xmotor);

   //Debug(xAngle,yAngle,(double)0);
   
  // Debug(balanceKp,balanceKi, balanceKd);   //4.5, 0.82 ,  6.8  good pid values w/ 3hz motor lowpass
   
   //Debug(Kp,Ki,Kd); 
  
  //Debug(Xlp,Ylp,(double)pot4);
  
  
}

//For Sabertooth 2x25 controller in simplified serial mode.
//Motor 1: 1=full reverse  64 = stop  127=full fwd
//Motor 2: 128=fill reverse  192=stop  255=full fwd
//Motor 1 is X, motor 2 is Y
void SetMotorSpeed(int x, int y)
{
  if (x > 126) x = 126;
  if (x < -126) x = -126;
  x = (x / 2) + 64;  //X Range now 1..127
   MotorSerial.print(x,BYTE);
   
  if(y > 126) y = 126;
  if(y < -126) y = -126;
  y = (y/2) + 192;  //Y Range now 129..255
  MotorSerial.print(y,BYTE);
  
}

void Debug(int db1,int db2, int db3)
{
  if(dbctr == 0) {
      Serial.print(db1);
      Serial.print("  ");
      Serial.print(db2);
      Serial.print("  ");
      Serial.println(db3);
      dbctr = 25;
  }
}

void Debug(float db1, float db2, float db3)
{
  if(dbctr == 0) {
      Serial.print(db1);
      Serial.print("  ");
      Serial.print(db2);
      Serial.print("  ");
      Serial.println(db3);
      dbctr = 25;
  }
}


  
  void timer100hz()
  { 
  
    if(dbctr > 0) dbctr--;
    if(timer1 > 0) timer1--;
    if(timer2 > 0) timer2--;
        
  }
  
  void readPots()
  {
    pot0 = muxPot(0);
    pot1 = muxPot(1);
    pot2 = muxPot(2);
    pot3 = muxPot(3);
    pot4 = analogRead(4);
  }
  
  int muxPot(int n)
  { byte i;
    if(n&1) digitalWrite(4, HIGH); else digitalWrite(4,LOW);
    if(n&2) digitalWrite(5, HIGH); else digitalWrite(5,LOW);
    if(n&4) digitalWrite(5, HIGH); else digitalWrite(6,LOW);
    for(i=0;i<50;i++) n = n;    //Waste some time
    return analogRead(5);
  }
  
  
  float limit(float value, float MIN, float MAX)
  {
      if(value > MAX) return MAX;
      if(value < MIN) return MIN;
      return value;
  }

