/*
 *  Lowpass1.cpp
 *  First order low pass filter library.
 *  Provide constructor with 3db cutoff frequency and sample time in milliseconds.
 *  
 *
 *  Created by Dale Heatherington on 8/30/11.
 * This Code is licensed under a Creative Commons Attribution-ShareAlike 3.0 Unported License. *
 */

#include <WProgram.h>
#include <Lowpass1.h>




/*Constructor for first order lowpass filter
 
 RC is time constant from equivalent analog filter  (resistance times capacitance) 
 Low pass 3 db cut off frequency = 1 / (2 * Pi * R * C)
 
 t is sample time interval in milliseconds  
 
 */
Lowpass1::Lowpass1(double FC, int t)
{   
	SampleTime = t;								//Set sample time in milliseonds
	double ts = (double)t / 1000.0;				//t is given in milliseconds.  Convert to seconds
	
	double RC = 0.1591549 / FC ;				//Compute time constant RC from cutoff frequency
	a = ts/(ts+RC);								//Compute Alpha from time constant
	lastTime = millis()-SampleTime;
	lastOutput = 0.0;
	
}

/* Lowpass Filter function.  
 This is called on every pass through the main loop.  
 It decides if it needs to execute based on sample time.
 */
double Lowpass1::filter(double input)
{   double output;
	unsigned long now = millis();
	int timeChange = (now - lastTime);
	
	if(timeChange>=SampleTime){				//Execute only every "SampleTime" seconds.
		
		output = (a*input) + ((1.00 - a) * lastOutput);  //  lowpass function.
		lastOutput = output;						//remember last value we computed
		lastTime = now;								// remember what time it is
	}
	
	return lastOutput;
}

